// arasan.cpp: Application class for Arasan Windows GUI.
// Copyright 1994, 2001, 2002 by Jon Dart. All Rights Reserved.

#include "stdafx.h"
#include "arasan.h"

#include "MainFrm.h"
#include "guiDoc.h"
#include "guiView.h"
#include "EngineCo.h"
#include "bearing.h"

#include <string.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Global variables
GuiOptions *guiOptions;
char initialFilename[MAX_PATH];

/////////////////////////////////////////////////////////////////////////////
// ArasanGuiApp

BEGIN_MESSAGE_MAP(ArasanGuiApp, CWinApp)
	//{{AFX_MSG_MAP(ArasanGuiApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_APP_EXIT, OnAppExit)
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()


static ThreadComm *comm = NULL;
char *programPath;

/////////////////////////////////////////////////////////////////////////////
// ArasanGuiApp construction

ArasanGuiApp::ArasanGuiApp()
{
}

/////////////////////////////////////////////////////////////////////////////
// The one and only ArasanGuiApp object

ArasanGuiApp theApp;

/////////////////////////////////////////////////////////////////////////////
// ArasanGuiApp initialization

BOOL ArasanGuiApp::InitInstance()
{
    // must do this first, many things depend on it:
    Bitmap::init();
    // set up attack bitmaps, etc.
    Bearing::init();

    // Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

  
    // Set the registry key.  This makes MFC store option data in
    // the Registry rather than in an .INI file.  Normally this
    // key is a company name - we don't have one, we use "Arasan".
	SetRegistryKey(_T("Arasan"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

    // load program-specific options
    guiOptions = new GuiOptions(this);

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(ArasanGuiDoc),
		RUNTIME_CLASS(ArasanMainFrame),       // main SDI frame window
		RUNTIME_CLASS(ArasanGuiView));
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

    char moduleName[MAX_PATH];
    GetModuleFileName(AfxGetInstanceHandle(),moduleName,MAX_PATH);

    // Set up a separate thread to handle communication with the
    // chess engine.
    comm = new ThreadComm;
    comm->app = this;
    comm->moduleName = strdup(moduleName);
	m_searchThread = AfxBeginThread(EngineThreadProc,comm);
 
    // Wait for the engine to start up.
    while (!engineReady()) WaitForSingleObject(m_searchThread,2000);

	// The one and only window has been initialized, so show and update it.
	//m_pMainWnd->ShowWindow(SW_SHOW);
	//m_pMainWnd->UpdateWindow();
    OnFileNew();

    POSITION pos = pDocTemplate->GetFirstDocPosition();
    ASSERT(pos);
    CDocument* firstDoc = pDocTemplate->GetNextDoc(pos);
    ASSERT(firstDoc);
    pos = firstDoc->GetFirstViewPosition();
    ASSERT(pos);
    theView = firstDoc->GetNextView(pos);
    ASSERT(theView);

    // Send the init string to the engine:
    init();

    // send time control string to engine:
    Search_Limit_Options options = guiOptions->get_src_limits();
    ((ArasanGuiView*)theView)->setTimeControl(options);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;
/**    *initialFilename = '\0';
    if (m_lpCmdLine[0] != '\0')
    {
       // command-line processing.
       int i = 0;
       // skip leading blanks, if any:
       while (m_lpCmdLine[i] != '\0' &&
              isspace(m_lpCmdLine[i])) ++i;
       int j = i;
       while (m_lpCmdLine[j] != '\0' &&
              !isspace(m_lpCmdLine[j])) ++j;
       m_lpCmdLine[j] = '\0'; // remove trailing blanks, if any
       strncpy(initialFilename,m_lpCmdLine+i,MAX_PATH);
    }**/
    return TRUE;
}

void ArasanGuiApp::init() {
    // The engine communication thread calls back to here
    // once the link is set up. 
    writeToEngine("protover 2");
    writeToEngine("new");
    writeToEngine("post");
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void ArasanGuiApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// ArasanGuiApp commands

void ArasanGuiApp::OnAppExit() 
{
    // TBD: May need to send "quit" twice:
    writeToEngine("quit\nquit\n");
    CWinApp::OnAppExit();	
}

int ArasanGuiApp::ExitInstance() 
{
    Bitmap::cleanup();	
	return CWinApp::ExitInstance();
}
