// Copyright 1994-2002 by Jon Dart. All Rights Reserved.
#ifndef _DISPLAY_H
#define _DISPLAY_H

#include <afxwin.h>
#include "board.h"
#include "stats.h"
#include "guiopts.h"

class Display {

public:
    Display(CWnd *parent, const CRect &size);

    virtual ~Display();

    void setSize(const CRect &);

    void resize(CDC *pDC, GuiOptions::BoardSize);

    void drawBoard( CDC *pDC, const Board &board, const CRect &drawArea, BOOL turned);

    void drawPiece( CDC *pDC, Square sq, Piece p);

    void showSide(CDC *pDC, ColorType side);

    void showTime(HWND hWin, time_t displayTime, ColorType side);

    void showECO(CDC *pDC, const char *eco, const char *openingName);

    void showMove(CDC *pDC, const string &moveImage, int moveCount, ColorType side);

    void showResult(CDC *pDC, const string &result);

    void showCoordinates(CDC *pDC);

    Square mouseLocation(const CPoint &point) const;

    void showHeader(CDC *pDC, const string &header);

    void setTurned(BOOL state) {
       turned = state;
    }

    BOOL isTurned() const {
       return turned;
    }

    CFont &getPieceFont()  {
        return pieceFont;
    }

    void setPieceFont(CDC *pDC, LPCSTR fontName, GuiOptions::BoardSize boardSize);

    static void calcWindowSize(GuiOptions::BoardSize boardSize, int &x, int &y);


protected:
    void setPieceFont( CDC *pDC, const LPLOGFONT font ) {
        pieceFont.Detach();
        pieceFont.CreateFontIndirect(font);
        updatePieceFont(pDC);
    }

    void updatePieceFont(CDC *pDC);

private:
     CBrush *get_brush( CDC *pDC, Square sq) const;
    
     DWORD char_width;
     //DWORD max_char_width;
     //DWORD char_height;
     static DWORD spacing;
     DWORD vertical_border_width;
     DWORD horiz_border_height;
     BOOL turned;
     BOOL mono;
     DWORD board_right_edge;
     DWORD width;
     DWORD textX;
     DWORD coordX;
     CFont pieceFont;
     CWnd *myWin;
};


#endif