// EngineCo.cpp : handles communication with the chess engine process
// Copyright 2001 by Jon Dart. All Rights Reserved.
//

#include "stdafx.h"
#include "arasan.h"
#include "EngineCo.h"
#include <ctype.h>
#include <string.h>
#include "mainfrm.h"

#include <iostream>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// EngineCommunication

static HANDLE hEngine,hWriteHandle;

static CRITICAL_SECTION queueCritSection;

static int queueSize = 0;
static const int QUEUE_MAX = 20;
static char *cmdQueue[QUEUE_MAX];
static BOOL ready = FALSE;
int force = 0;

BOOL engineReady() {
   return ready;
}

int commandCount()
{
   EnterCriticalSection(&queueCritSection);
   int size = queueSize;
   LeaveCriticalSection(&queueCritSection);
   return size;
}

// read and remove a command from the pending queue
char * readCommand() {
   char *retVal;
   EnterCriticalSection(&queueCritSection);
   if (queueSize) {
     retVal = strdup(cmdQueue[0]);
     free(cmdQueue[0]);
     for (int i = 0; i < queueSize-1; i++) {
        cmdQueue[i] = cmdQueue[i+1];
     }
     --queueSize;
   }
   else
     retVal = NULL;  
   LeaveCriticalSection(&queueCritSection);
   return retVal;
}

void pushCommand(LPCSTR cmd) {
   EnterCriticalSection(&queueCritSection);
   ASSERT(queueSize < QUEUE_MAX);
   cmdQueue[queueSize++] = strdup(cmd);
   LeaveCriticalSection(&queueCritSection);
}


static void DisplayError(LPCSTR function) {
   LPVOID errMsg;

   FormatMessage(
                FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
                NULL,
                GetLastError(),
                MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                (LPTSTR)&errMsg,
                0,
                NULL 
                );
   TRACE("failure starting engine: %s: %s\n",function,errMsg);
   LocalFree(errMsg);
      
}
HANDLE startProcess(char *cmdLine, HANDLE hChildStdOut,HANDLE hChildStdIn, HANDLE hChildStdErr)
{
    BOOL result;
    STARTUPINFO si;
    PROCESS_INFORMATION pi;

    ZeroMemory(&si,sizeof(STARTUPINFO));    
    si.cb = sizeof(STARTUPINFO);
    ZeroMemory(&pi,sizeof(PROCESS_INFORMATION));
    si.dwFlags = STARTF_USESTDHANDLES;
    si.hStdOutput = hChildStdOut;
    si.hStdInput  = hChildStdIn;
    si.hStdError  = hChildStdErr;
    si.wShowWindow = SW_HIDE;
      
    TRACE("calling CreateProcess with command line %s\n",cmdLine);
    result = CreateProcess(
                NULL,
                cmdLine, // executable path + any arguments
                NULL, // security attributes
                NULL, // thread attributes
                TRUE, // bInheritHandles
#ifdef _DEBUG
                0, 
#else
                CREATE_NO_WINDOW | DETACHED_PROCESS | CREATE_NEW_PROCESS_GROUP,     // creation flags
#endif
                NULL,  // environment block
                NULL,  // current directory
                &si,
                &pi);
    
    if (result)
    {
        // detach from the child process
        CloseHandle(pi.hThread);
        //CloseHandle(pi.hProcess);
        return pi.hProcess;
    }
    else
    {
       DisplayError("CreateProcess");
       return NULL;
    }
}


inline CString GetLastErrorText(BOOL bShowMessageBoxAlso = TRUE)
{
	LPVOID lpMsgBuf;
	FormatMessage( FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
		NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
		(LPTSTR) &lpMsgBuf, 0, NULL );

	CString cstrMessageText = (LPTSTR)lpMsgBuf;

	if(bShowMessageBoxAlso == TRUE)
	{
		// Display the string.
		MessageBox( NULL, cstrMessageText, _T("GetLastErrorText"), MB_OK|MB_ICONINFORMATION );
	}
	// Free the buffer.
	LocalFree( lpMsgBuf );
	return cstrMessageText;
}


#define ErrorExit(msg) { comm->errorMsg=strdup(msg); \
   comm->result = GetLastError(); \
   GetLastErrorText(); \
   return -1;}


UINT CDECL EngineThreadProc( LPVOID pParam )
{
    HANDLE hOutputReadTmp,hOutputRead,hOutputWrite;
    HANDLE hInputWriteTmp,hInputRead,hInputWrite;
    HANDLE hErrorWrite;
      
    InitializeCriticalSection(&queueCritSection);

    ThreadComm *comm = (ThreadComm*)pParam;
	char *moduleName = comm->moduleName;
    char app_path[MAX_PATH];
    strcpy(app_path,moduleName);
    // We assume the chess engine is located in the same directory as
    // the GUI executable.
    // Find the last part of the program pathname:
    char *p = strrchr(app_path,'\\');
    if (p)
    {
        // replace it with the engine name
        strcpy(p+1,"arasanx.exe");
    }
    else
       strcpy(app_path,"arasanx.exe");
#ifdef _DEBUG
    strcat(app_path," -t");
#endif

    SECURITY_ATTRIBUTES sa; 
 
    // Redirect child handles. See Microsoft Support article Q190351

      // Set up the security attributes struct.
      sa.nLength= sizeof(SECURITY_ATTRIBUTES);
      sa.lpSecurityDescriptor = NULL;
      sa.bInheritHandle = TRUE;

      // Create the child output pipe.
      if (!CreatePipe(&hOutputReadTmp,&hOutputWrite,&sa,0))
         DisplayError("CreatePipe");

      // Create a duplicate of the output write handle for the std error
      // write handle. This is necessary in case the child application
      // closes one of its std output handles.
      if (!DuplicateHandle(GetCurrentProcess(),hOutputWrite,
                           GetCurrentProcess(),&hErrorWrite,0,
                           TRUE,DUPLICATE_SAME_ACCESS))
         DisplayError("DuplicateHandle");


      // Create the child input pipe.
      if (!CreatePipe(&hInputRead,&hInputWriteTmp,&sa,0))
         DisplayError("CreatePipe");


      // Create new output read handle and the input write handles. Set
      // the Properties to FALSE. Otherwise, the child inherits the
      // properties and, as a result, non-closeable handles to the pipes
      // are created.
      if (!DuplicateHandle(GetCurrentProcess(),hOutputReadTmp,
                           GetCurrentProcess(),
                           &hOutputRead, // Address of new handle.
                           0,FALSE, // Make it uninheritable.
                           DUPLICATE_SAME_ACCESS))
         DisplayError("DuplicateHandle");

      if (!DuplicateHandle(GetCurrentProcess(),hInputWriteTmp,
                           GetCurrentProcess(),
                           &hInputWrite, // Address of new handle.
                           0,FALSE, // Make it uninheritable.
                           DUPLICATE_SAME_ACCESS))
      DisplayError("DuplicateHandle");


      // Close inheritable copies of the handles you do not want to be
      // inherited.
      if (!CloseHandle(hOutputReadTmp)) DisplayError("CloseHandle");
      if (!CloseHandle(hInputWriteTmp)) DisplayError("CloseHandle");
      if (!CloseHandle(hErrorWrite)) DisplayError("CloseHandle");
 
    //comm->app->setEngineHandle(pHandle);

    HANDLE hProcess = startProcess(app_path,hOutputWrite,hInputRead,hErrorWrite);

    hWriteHandle = hInputWrite;
    ready = TRUE;

    //WaitForSingleObject(hChildStdoutRdDup,2000);

    // big buffer may be needed if engine has debugging on.
    const int BUFSIZE = 16384;
	char chBuf[BUFSIZE];
	DWORD dwRead = 0;
    int bufLen = 0;
    int offset = 0;

    // Now we're ready to communicate. Wait for engine input.
    for (;;) 
    { 
      if( !ReadFile( hOutputRead, chBuf+bufLen, BUFSIZE, &dwRead, 
         NULL) || dwRead == 0) break; 
      // split into lines.
      char cmdBuf[256];
      char *cmd = cmdBuf;
      char *buf = chBuf;
      bufLen += dwRead;
      
      int n = 0;
      for (;;) {
         while (n < bufLen && chBuf[n] && chBuf[n] != '\012' && chBuf[n] != '\015') {
            //TRACE("got char: %d\n",(int)chBuf[n]);
            *cmd++ = chBuf[n++];
         }
         if (chBuf[n] == '\012' || chBuf[n] == '\015') {
            *cmd = '\0';
            if (strlen(cmdBuf) != 0) {
              TRACE("got cmd: %s\n",cmdBuf);
              pushCommand(cmdBuf);
              if (comm->app->getView())
                  comm->app->getView()->SendMessage(WM_COMMAND,ID_ENGINE_REPLY,0L);
              else {
                 TRACE("GUI not ready: could not dispatch command\n");

              }
            }
            cmd = cmdBuf;
            bufLen -= (n+1);
            if (bufLen <= 0) break;
            int i = 0; 
            while (i < bufLen) {
               chBuf[i] = chBuf[i+n+1];
               ++i;
            }
            chBuf[i] = '\0';
            n = 0;
         }
         else {
            // stuff left but it's not a complete line
             TRACE("incomplete line, breaking\n");
            break;
         }
      }
    } 
    TRACE("exiting read thread");
	return 0;
}

void writeToEngine(char *cmd) {
 
    if (ready == FALSE) {
       TRACE("ignoring command: \"%s\" - not ready yet\n",cmd);
       return;
    }
    DWORD written;
    char buf[255];
    strcpy(buf,cmd);
    if (buf[strlen(buf)-1] != '\n')
      strcat(buf,"\n");
    TRACE("sending to engine: %s",buf);
 
    if (!::WriteFile(hWriteHandle,buf,strlen(buf),&written,NULL)) {
       DisplayError("WriteFile"); 
    }
}

void setForce(int value) {
   if (value) {
     if (!force) {
       writeToEngine("force\n");
       force = 1;
       ArasanMainFrame *mainFrm = (ArasanMainFrame*)AfxGetMainWnd();
       if (mainFrm) mainFrm->setStatus(ArasanMainFrame::Ready);
     }
  }
  else if (force) {
     writeToEngine("go\n");
     force = 0;
  }
}

void startSearch() {
   writeToEngine("go");
   force = 0;
}
