// Copyright 1996, 1997, 2001, 2002 by Jon Dart.  All Rights Reserved.

#include "globals.h"
#include "bearing.h"
#include "moveord.h"
#include "scoring.h"
#include "pinfo.h"
#include "config.h"

#ifdef TABLEBASES
#include "tbprobe.h"
#endif

char cmd_buf[256];
int cmd_buf_count;

Book *opening_book = NULL;
PositionBook *position_book = NULL;
Log *theLog = NULL;
int hash_table_size = 8000;
MoveArray *game_moves;
Options options;
char *programPath;
int EGTBMenCount;

void init_globals(const char *pathName, 
		  int useOpeningBook, int createLog, int useTB)
{
    // must do this first, many things depend on it:
    Bitmap::init();
    // set up attack bitmaps, etc.
    Bearing::init();
    programPath = new char[MAX_PATH];
    strcpy(programPath,pathName);

    char rcPath[MAX_PATH];
    strcpy(rcPath,pathName);
#ifdef _WIN32
    char *p = strrchr(rcPath,'\\');
#else
    char *p = strrchr(rcPath,'/');
#endif
    if (p == NULL) 
      strcpy(rcPath,"arasan.rc");
    else
      strcpy(p+1,"arasan.rc");
    // try to read arasan.rc file
    options.init(rcPath);

#ifdef TABLEBASES
     // If no tablebase directory was specified in the .rc file,
     // try to find the table bases under the arasan install dir
    if (options.search.use_tablebases &&
        options.search.tablebase_location == "") {
    char tbPath[MAX_PATH];
    strcpy(tbPath,pathName);
     const char *dir = "TB";
#ifdef _MSC_VER
     char *p = strrchr(tbPath,'\\');
#else
     char *p = strrchr(tbPath,'/');
#endif
      if (p)
      {
        // replace it with the tb dir name
	    strcpy(p+1,dir);
      }
      else
         strcpy(tbPath,dir);
      options.search.tablebase_location=tbPath;
    }
#endif
    cmd_buf_count = 0;
    *cmd_buf = '\0';
    game_moves = new MoveArray();
    if (createLog)
    {
       theLog = new Log();
       theLog->clear();
       theLog->write_header();
    }
    if (useOpeningBook)
    {
       opening_book = new Book();
       position_book = new PositionBook();
    }

    Move_Ordering::clear_killer();
    Move_Ordering::clear_history();
#ifdef TABLEBASES
    if (useTB) {
       EGTBMenCount = initTB((char*)options.search.tablebase_location.c_str(),
         options.search.tb_cache_size);
    }
#endif    
    Board b;
}

// Make this a __cdecl function so we can pass it to atexit()
void CDECL cleanup_globals()
{
    delete opening_book;
    delete position_book;
    delete game_moves;
    if (theLog) delete theLog;
    delete [] programPath;
    Scoring::cleanup();
    Bitmap::cleanup();
}
