// Copyright 1993, 1995, 1999 by Jon Dart. All Rights Reserved.

#include "pinfo.h"
#include "debug.h"

static const hash_t rep_codes[3] =
{
    MAKELONGLONG(0x194ca2c4,0x5c8e7baa),
    MAKELONGLONG(0x804e48e8,0xe8f5544f),
    MAKELONGLONG(0xd4767986,0xf0ab49a7)
};

Position_Info::Position_Info()
{
}

extern const hash_t rep_codes[3];

Position_Info::Position_Info( const Board &board, int rep_count )
: my_hashcode(board.HashCode())
{
   ASSERT(rep_count < 3);
   my_hashcode ^= rep_codes[rep_count];
}

Position_Info::Position_Info( const Board &board, const int depth,
                              const byte age,
	           	      const ValueType hashflags,
			      const int forced,
                              const int in_check,
			      const int rep_count,
			      const int value,
			      const Move &best_move)
:  my_hashcode(board.HashCode()),
   my_depth(depth),
   my_age(age),
   my_value(value),start(StartSquare(best_move)),
   dest(DestSquare(best_move))
{
    ASSERT(rep_count < 3);
    my_hashcode ^= rep_codes[rep_count];
    my_depth |= ((uint16)PromoteTo(best_move)) << 11;
    my_flags = (byte)hashflags;
    if (forced)
        my_flags |= 0x80;
    if (in_check)
        my_flags |= 0x40;
}

Move Position_Info::best_move(const Board &b) const
{
    if (start == InvalidSquare)
       return NullMove;
    else
       return CreateMove(b,(Square)start,(Square)dest,
         (PieceType)(my_depth>>11));
}

