// Copyright 1992 by Jon Dart.  All Rights Reserved.

#ifndef _SCORING_H
#define _SCORING_H

#include "board.h"
class King_Pawn_Entry;

class Scoring
{
	// This class does static evaluation of chess positions.

	public:
		
	static void init(const Board &board);
        
        static void cleanup();
		
	// evaluate "board" from the perspective of the side to move.
	static int evalu8( const Board &board );
		
	// return a positional score
	static int positional_score( const Board &board, int alpha, int beta );

	// return a material score
	static int material_score( const Board &board );
		
	// return what the material score will be after move "m"
    // is made. board holds the position before the move is 
    // made. Score is from the perspective of the current
    // side to move.
	static int material_score( const Board &board, Move m );

    static int in_endgame( const Board &board );

    static int repetition_draw( const Board &board );

    static int material_draw( const Board &board );
        
    // checks for legal draws plus certain other theoretically
    // draw positions
    static int is_draw(const Board &board);
        
    // Check only for those positions which are not legal draws
    // but are theoretically drawn
    static int is_theoretical_draw(const Board &board);
        
    // Check only for legal draws
    static int is_legal_draw(const Board &board);

    // Turn a score into a formatted string (mate scores are
    // show like +Mate6).
    static const char* print_score( const int score );

protected:
        static int positional_score( const Board &, const ColorType );

};

#endif
